#include "Object.h"
#include <iostream>
#include <sstream>
#include <string>

Room::Room(RoomType type)
{
	Type = type;
	Rect = sf::FloatRect(0,0,1,1);
}
Room::Room(RoomType type, sf::FloatRect rect)
{
	Type = type;
	Rect = rect;
}

Object::Object()
{
	ShufflePosition();
	m_Template = NULL;
	m_PlayerID = 0;
	m_VisibleToPlayer = 1.0;
	m_Visible = true;
	m_Alive = true;
}

void Object::ShufflePosition()
{
	m_Position = sf::Vector2f( rand()%490,rand()%240 );
	while (!TryPosition(m_Position))
	{
		m_Position = sf::Vector2f( rand()%490,rand()%240 );
	}
}

Object* Object::Spawn()
{
	Object* new_Obj = new Object();
	
	new_Obj->m_Rooms = m_Rooms;
	new_Obj->m_KnownEnemies = m_KnownEnemies;
	new_Obj->m_Template = m_Template;
	new_Obj->m_PlayerID = m_PlayerID;
	new_Obj->m_VisibleToPlayer = 1.0;
	
	std::stringstream ss;
	ss << m_Name;
	ss << rand()%999;
	new_Obj->m_Name = ss.str();
	
	new_Obj->ShufflePosition();
	return new_Obj;
}

bool Object::TryPosition(sf::Vector2f trypos)
{
	trypos = trypos + sf::Vector2f(-10,-10);
	
	sf::FloatRect fr;
	fr = sf::FloatRect(20,28,146,20);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(79,48,73,27);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(151,48,28,16);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(88,76,53,18);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(105,95,38,21);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(145,113,42,40);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(166,129,35,31);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(169,161,16,15);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(154,155,15,51);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(148,216,23,19);
	if (fr.contains(trypos)) return true;
	
	fr = sf::FloatRect(240,44,19,12);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(259,29,250,27);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(335,18,143,11);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(242,57,217,19);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(230,76,107,44);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(271,117,30,58);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(302,118,16,36);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(336,77,93,20);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(356,96,20,19);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(389,97,89,51);
	if (fr.contains(trypos)) return true;
	fr = sf::FloatRect(415,148,59,35);
	if (fr.contains(trypos)) return true;

	return false;
}

char RoomTypeToChar( RoomType rt)
{
	switch (rt)
	{
		case PLAYERLAIR:
			return '@';
		case PLAYERBASE:
			return '@';
		case LOOK:
			return 'l';
		case FORAGE:
			return 'f';
		case ROCK:
			return 'R';
		case BPAPER:
			return 'B';
		case XSCISSORS:
			return 'X';
		case TEMPLATE:
			return 'T';
		case CREATE:
			return 'C';
		case PING:
			return 'p';
		case FRIENDPING:
			return 'F';
		case RELAY:
			return 'r';
		case WFRIENDRELAY:
			return 'w';
		case SNOOP:
			return 's';
		case ASCREAM:
			return 'A';
		case ASEEKSCREAM:
			return 'a';
		case KCHARGE:
			return 'k';
		case MEANDER:
			return 'm';
		case DECOY:
			return 'd';
		case PINGFAKE:
			return 'b';
		default:
			return '0';
	}
}

RoomType CharToRoomType(char ch)
{
	switch (ch)
	{
		case '@':
			return PLAYERBASE;
		case 'l':
			return LOOK;
		case 'f':
			return FORAGE;
		case 'R':
			return ROCK;
		case 'B':
			return BPAPER;
		case 'X':
			return XSCISSORS;
		case 'T':
			return TEMPLATE;
		case 'C':
			return CREATE;
		case 'p':
			return PING;
		case 'F':
			return FRIENDPING;
		case 'r':
			return RELAY;
		case 'w':
			return WFRIENDRELAY;
		case 's':
			return SNOOP;
		case 'A':
			return ASCREAM;
		case 'a':
			return ASEEKSCREAM;
		case 'k':
			return KCHARGE;
		case 'm':
			return MEANDER;
		case 'd':
			return DECOY;
		case 'b':
			return PINGFAKE;
		default:
			return PING;
	}
}


std::string RoomTypeToName(RoomType rt)
{
	switch (rt)
	{
		case PLAYERLAIR:
			return "@Playerlair";
		case PLAYERBASE:
			return "@Playerbase";
		case LOOK:
			return "look";
		case FORAGE:
			return "forage";
		case ROCK:
			return "Rock";
		case BPAPER:
			return "BPaper";
		case XSCISSORS:
			return "XScissors";
		case TEMPLATE:
			return "Template";
		case CREATE:
			return "Create";
		case PING:
			return "ping";
		case FRIENDPING:
			return "Friendping";
		case RELAY:
			return "relay";
		case WFRIENDRELAY:
			return "wfriendrelay";
		case SNOOP:
			return "snoop";
		case ASCREAM:
			return "Ascream";
		case ASEEKSCREAM:
			return "aseekscream";
		case KCHARGE:
			return "kCharge";
		case MEANDER:
			return "meander";
		case DECOY:
			return "decoy";
		case PINGFAKE:
			return "bpingfake";
		default:
			return "";
	}
}